@@ -39,8 +39,11 @@ struct _ClipmanCollectorPrivate
   ClipmanHistory       *history;
   GtkClipboard         *default_clipboard;
   GtkClipboard         *primary_clipboard;
+  gchar                *default_cache;
+  gchar                *primary_cache;
   guint                 primary_clipboard_timeout;
-  gboolean              internal_change;
+  gboolean              default_internal_change;
+  gboolean              primary_internal_change;
   gboolean              add_primary_clipboard;
   gboolean              persistent_primary_clipboard;
   gboolean              history_ignore_primary_clipboard;
@@ -98,6 +101,14 @@ cb_check_primary_clipboard (gpointer user_data)
 
   g_return_val_if_fail (GTK_IS_CLIPBOARD (collector->priv->default_clipboard) && GTK_IS_CLIPBOARD (collector->priv->primary_clipboard), FALSE);
 
+  /* Jump over if the content is set from within clipman */
+  if (collector->priv->primary_internal_change)
+    {
+      collector->priv->primary_internal_change = FALSE;
+      collector->priv->primary_clipboard_timeout = 0;
+      return FALSE;
+    }
+
   /* Postpone until the selection is done */
   gdk_window_get_device_position (root_win, device, NULL, NULL, &state);
   if (state & (GDK_BUTTON1_MASK|GDK_SHIFT_MASK))
@@ -126,16 +137,16 @@ cb_clipboard_owner_change (ClipmanCollector *collector,
       return;
     }
 
-  /* Jump over if the content is set from within clipman */
-  if (collector->priv->internal_change)
-    {
-      collector->priv->internal_change = FALSE;
-      return;
-    }
-
   /* Save the clipboard content to ClipmanHistory */
   if (event->selection == GDK_SELECTION_CLIPBOARD)
     {
+      /* Jump over if the content is set from within clipman */
+      if (collector->priv->default_internal_change)
+        {
+          collector->priv->default_internal_change = FALSE;
+          return;
+        }
+
       if (gtk_clipboard_wait_is_image_available (collector->priv->default_clipboard))
         {
           image = gtk_clipboard_wait_for_image (collector->priv->default_clipboard);
@@ -145,6 +156,7 @@ cb_clipboard_owner_change (ClipmanCollector *collector,
         }
       else
         {
+          clipman_history_set_image_to_restore (collector->priv->history, NULL);
           gtk_clipboard_request_text (collector->priv->default_clipboard,
                                       (GtkClipboardTextReceivedFunc)cb_request_text,
                                       collector);
@@ -161,9 +173,13 @@ cb_clipboard_owner_change (ClipmanCollector *collector,
           || !collector->priv->history_ignore_primary_clipboard
           || collector->priv->enable_actions)
         {
-          if (collector->priv->primary_clipboard_timeout == 0)
-            collector->priv->primary_clipboard_timeout =
-              g_timeout_add (250, cb_check_primary_clipboard, collector);
+          if (collector->priv->primary_clipboard_timeout != 0)
+            {
+              g_source_remove (collector->priv->primary_clipboard_timeout);
+              collector->priv->primary_clipboard_timeout = 0;
+            }
+          collector->priv->primary_clipboard_timeout =
+            g_timeout_add (250, cb_check_primary_clipboard, collector);
         }
     }
 }
@@ -173,52 +189,77 @@ cb_request_text (GtkClipboard *clipboard,
                  const gchar *text,
                  ClipmanCollector *collector)
 {
-  static gchar *prev_text = NULL;
-
   g_return_if_fail (GTK_IS_CLIPBOARD (collector->priv->default_clipboard) && GTK_IS_CLIPBOARD (collector->priv->primary_clipboard));
 
-  if (text == NULL || text[0] == '\0')
+  if (text == NULL)
     {
       /* Restore primary clipboard on deselection */
-      if (clipboard == collector->priv->primary_clipboard
-          && collector->priv->persistent_primary_clipboard
-          && prev_text != NULL)
-        gtk_clipboard_set_text (collector->priv->primary_clipboard, prev_text, -1);
+      if (clipboard == collector->priv->primary_clipboard && collector->priv->primary_cache != NULL && (
+            (collector->priv->persistent_primary_clipboard && !collector->priv->add_primary_clipboard)
+            || (collector->priv->add_primary_clipboard
+                && gtk_clipboard_wait_is_text_available (collector->priv->default_clipboard))
+         ))
+        {
+          collector->priv->primary_internal_change = TRUE;
+          gtk_clipboard_set_text (collector->priv->primary_clipboard, collector->priv->primary_cache, -1);
+        }
+
+      /* Restore default clipboard when cleared unintentionally (if this was intentional,
+       * 'default_cache' should have been cleared before) */
+      if (clipboard == collector->priv->default_clipboard && collector->priv->default_cache != NULL)
+        {
+          collector->priv->default_internal_change = TRUE;
+          gtk_clipboard_set_text (collector->priv->default_clipboard, collector->priv->default_cache, -1);
+        }
 
       return;
     }
 
   if (clipboard == collector->priv->default_clipboard)
     {
+      /* Add to history */
       clipman_history_add_text (collector->priv->history, text);
+
+      /* Make a copy inside the primary clipboard */
+      if (collector->priv->add_primary_clipboard && g_strcmp0 (text, collector->priv->primary_cache) != 0)
+        {
+          collector->priv->primary_internal_change = TRUE;
+          gtk_clipboard_set_text (collector->priv->primary_clipboard, text, -1);
+          g_free (collector->priv->primary_cache);
+          collector->priv->primary_cache = g_strdup (text);
+        }
+
+      /* Match for actions */
       if (collector->priv->enable_actions)
         clipman_actions_match_with_menu (collector->priv->actions, ACTION_GROUP_MANUAL, text);
+
+      g_free (collector->priv->default_cache);
+      collector->priv->default_cache = g_strdup (text);
     }
   else if (clipboard == collector->priv->primary_clipboard)
     {
       /* Avoid history */
-      if (collector->priv->add_primary_clipboard && collector->priv->history_ignore_primary_clipboard)
-        collector->priv->internal_change = TRUE;
-      else if (!collector->priv->history_ignore_primary_clipboard)
+      if (!collector->priv->history_ignore_primary_clipboard)
         clipman_history_add_text (collector->priv->history, text);
 
       /* Make a copy inside the default clipboard */
-      if (collector->priv->add_primary_clipboard)
-        gtk_clipboard_set_text (collector->priv->default_clipboard, text, -1);
-
-      /* Store selection for later use */
-      if (collector->priv->persistent_primary_clipboard)
+      if (collector->priv->add_primary_clipboard && g_strcmp0 (text, collector->priv->default_cache) != 0)
         {
-          g_free (prev_text);
-          prev_text = g_strdup (text);
+          collector->priv->default_internal_change = TRUE;
+          gtk_clipboard_set_text (collector->priv->default_clipboard, text, -1);
+          g_free (collector->priv->default_cache);
+          collector->priv->default_cache = g_strdup (text);
         }
 
       /* Match for actions */
-      if (collector->priv->enable_actions && g_strcmp0 (text, prev_text))
+      if (collector->priv->enable_actions)
+        clipman_actions_match_with_menu (collector->priv->actions, ACTION_GROUP_SELECTION, text);
+
+      /* Store selection for later use */
+      if (collector->priv->persistent_primary_clipboard || collector->priv->add_primary_clipboard)
         {
-          clipman_actions_match_with_menu (collector->priv->actions, ACTION_GROUP_SELECTION, text);
-          g_free (prev_text);
-          prev_text = g_strdup (text);
+          g_free (collector->priv->primary_cache);
+          collector->priv->primary_cache = g_strdup (text);
         }
     }
 }
@@ -231,17 +272,28 @@ cb_request_text (GtkClipboard *clipboard,
  * clipman_collector_set_is_restoring:
  * @collector: a #ClipmanCollector
  *
- * Call this function before modifing the content of a #GtkClipboard so that
+ * Call this function before modifying the content of a #GtkClipboard so that
  * the new content won't be looked by #ClipmanCollector.  Useful to prevent an
- * image from being saved twice.  Note that for text this is useless because
- * texts are compared, and a match throws the text in the history to the top of
- * the list.
- * See also clipman_history_set_item_to_restore().
+ * image from being saved twice.
+ * See also clipman_history_set_image_to_restore().
  */
 void
-clipman_collector_set_is_restoring (ClipmanCollector *collector)
+clipman_collector_set_is_restoring (ClipmanCollector *collector,
+                                    GtkClipboard *clipboard)
 {
-  collector->priv->internal_change = TRUE;
+  if (clipboard == collector->priv->default_clipboard)
+    collector->priv->default_internal_change = TRUE;
+  else if (clipboard == collector->priv->primary_clipboard)
+    collector->priv->primary_internal_change = TRUE;
+}
+
+void
+clipman_collector_clear_cache (ClipmanCollector *collector)
+{
+  g_free (collector->priv->default_cache);
+  g_free (collector->priv->primary_cache);
+  collector->priv->default_cache = NULL;
+  collector->priv->primary_cache = NULL;
 }
 
 ClipmanCollector *
@@ -260,43 +312,6 @@ clipman_collector_get (void)
   return singleton;
 }
 
-void
-clipman_collector_show_actions (void)
-{
-  ClipmanCollector   *collector;
-  const ClipmanHistoryItem *item;
-  ClipmanHistory     *history;
-  GSList *entries;
-  gint group;
-
-  history = clipman_history_get();
-  collector = clipman_collector_get();
-
-  item = clipman_history_get_item_to_restore(history);
-
-  if (item == NULL)
-    return;
-
-  if (item->type == CLIPMAN_HISTORY_TYPE_TEXT)
-    {
-      entries = clipman_actions_match (collector->priv->actions, ACTION_GROUP_MANUAL, item->content.text);
-      if (entries == NULL)
-        {
-          group = ACTION_GROUP_SELECTION;
-        }
-      else
-        {
-          group = ACTION_GROUP_MANUAL;
-        }
-      g_slist_free (entries);
-      clipman_actions_match_with_menu (collector->priv->actions, group, item->content.text);
-    }
-  else
-    {
-      DBG("I can't show actions on item type %d", item->type);
-    }
-}
-
 /*
  * GObject
  */
@@ -355,9 +370,10 @@ clipman_collector_init (ClipmanCollector *collector)
 {
   collector->priv = clipman_collector_get_instance_private (collector);
 
-  /* This bit is set to TRUE when a clipboard has to be set from within clipman
-   * while avoiding to re-add it to the history. */
-  collector->priv->internal_change = FALSE;
+  /* This bit is set to TRUE when a clipboard is set from within clipman to avoid
+   * re-adding to the history or triggering actions several times */
+  collector->priv->default_internal_change = FALSE;
+  collector->priv->primary_internal_change = FALSE;
 
   /* ClipmanActions */
   collector->priv->actions = clipman_actions_get ();
@@ -375,17 +391,25 @@ clipman_collector_constructed (GObject *object)
 {
   ClipmanCollector *collector = CLIPMAN_COLLECTOR (object);
 
-  g_signal_connect_swapped (collector->priv->default_clipboard, "owner-change",
-                            G_CALLBACK (cb_clipboard_owner_change), collector);
-  g_signal_connect_swapped (collector->priv->primary_clipboard, "owner-change",
-                            G_CALLBACK (cb_clipboard_owner_change), collector);
+  g_signal_connect_object (collector->priv->default_clipboard, "owner-change",
+                           G_CALLBACK (cb_clipboard_owner_change), collector, G_CONNECT_SWAPPED);
+  g_signal_connect_object (collector->priv->primary_clipboard, "owner-change",
+                           G_CALLBACK (cb_clipboard_owner_change), collector, G_CONNECT_SWAPPED);
 }
 
 static void
 clipman_collector_finalize (GObject *object)
 {
   ClipmanCollector *collector = CLIPMAN_COLLECTOR (object);
+
+  if (collector->priv->primary_clipboard_timeout != 0)
+    {
+      g_source_remove (collector->priv->primary_clipboard_timeout);
+      collector->priv->primary_clipboard_timeout = 0;
+    }
+  g_object_unref (collector->priv->actions);
   g_object_unref (collector->priv->history);
+  clipman_collector_clear_cache (collector);
 }
 
 static void
