@@ -25,17 +25,6 @@
 #ifndef __CLIPMAN_COMMON_H__
 #define __CLIPMAN_COMMON_H__
 
-/* G_GNUC_FALLTHROUGH definition, from GLib 2.68.0 */
-#if ! GLIB_CHECK_VERSION (2, 60, 0)
-#if __GNUC__ > 6
-#define G_GNUC_FALLTHROUGH __attribute__((fallthrough))
-#elif g_macro__has_attribute (fallthrough)
-#define G_GNUC_FALLTHROUGH __attribute__((fallthrough))
-#else
-#define G_GNUC_FALLTHROUGH
-#endif
-#endif
-
 #define DEFAULT_MAX_MENU_ITEMS                          15
 #define DEFAULT_MAX_TEXTS_IN_HISTORY                    100
 #define DEFAULT_MAX_IMAGES_IN_HISTORY                   1
@@ -45,6 +34,7 @@
 #define DEFAULT_REORDER_ITEMS                           TRUE
 #define DEFAULT_SKIP_ACTION_ON_KEY_DOWN                 FALSE
 #define DEFAULT_ADD_PRIMARY_CLIPBOARD                   FALSE
+#define DEFAULT_PERSISTENT_PRIMARY_CLIPBOARD            FALSE
 #define DEFAULT_SHOW_QR_CODE                            FALSE
 #define DEFAULT_HISTORY_IGNORE_PRIMARY_CLIPBOARD        TRUE
 #define DEFAULT_ENABLE_ACTIONS                          FALSE
@@ -73,5 +63,6 @@
 
 void                    clipman_common_show_info_dialog    (void);
 void                    clipman_common_show_warning_dialog (void);
+gchar *                 clipman_common_shorten_preview     (const gchar *text);
 
 #endif /* !__CLIPMAN_COMMON_H__ */
