@@ -36,7 +36,7 @@
 struct _ClipmanHistoryPrivate
 {
   GSList                       *items;
-  const ClipmanHistoryItem     *item_to_restore;
+  const ClipmanHistoryItem     *image_to_restore;
   guint                         max_texts_in_history;
   guint                         max_images_in_history;
   gboolean                      save_on_quit;
@@ -180,7 +180,6 @@ _clipman_history_add_item (ClipmanHistory *history,
 
   /* Prepend item to start of the history */
   history->priv->items = g_slist_prepend (history->priv->items, item);
-  history->priv->item_to_restore = item;
 
   /* Emit signal */
   g_signal_emit (history, signals[ITEM_ADDED], 0);
@@ -258,6 +257,9 @@ clipman_history_add_text (ClipmanHistory *history,
   ClipmanHistoryItem *item;
   GSList *list;
 
+  if (text == NULL || *text == '\0')
+    return;
+
   /* Search for a previously existing content */
   list = g_slist_find_custom (history->priv->items, text, (GCompareFunc)__g_slist_compare_texts);
   if (list != NULL)
@@ -271,7 +273,6 @@ clipman_history_add_text (ClipmanHistory *history,
         }
       else
         {
-          history->priv->item_to_restore = item;
           return;
         }
     }
@@ -316,6 +317,7 @@ clipman_history_add_image (ClipmanHistory *history,
   DBG ("Copy of image (%p) is (%p)", image, item->content.image);
 
   _clipman_history_add_item (history, item);
+  history->priv->image_to_restore = item;
 }
 
 /**
@@ -347,21 +349,22 @@ clipman_history_get_max_texts_in_history (ClipmanHistory *history)
 }
 
 /**
- * clipman_history_get_item_to_restore:
+ * clipman_history_get_image_to_restore:
  * @history: a #ClipmanHistory
  *
- * Returns the most recent item that has been added to #ClipmanHistory.
+ * Returns the image to restore, set via clipman_history_set_image_to_restore()
+ * or when adding a new image to history via clipman_history_add_image()
  *
  * Returns: a #const #ClipmanHistoryItem
  */
 const ClipmanHistoryItem *
-clipman_history_get_item_to_restore (ClipmanHistory *history)
+clipman_history_get_image_to_restore (ClipmanHistory *history)
 {
-  return history->priv->item_to_restore;
+  return history->priv->image_to_restore;
 }
 
 /**
- * clipman_history_set_item_to_restore:
+ * clipman_history_set_image_to_restore:
  * @history: a #ClipmanHistory
  * @item: a #ClipmanHistoryItem that must exist inside #ClipmanHistory
  *
@@ -370,16 +373,18 @@ clipman_history_get_item_to_restore (ClipmanHistory *history)
  * must be set as being restored with clipman_collector_set_is_restoring(),
  * than this function is called with the item that contains the image that is
  * getting restored.
- * Instead of being destroyed/recreated inside the history, it will remain at
- * the same position in the history (unlike being pushed to the top) but will
- * be marked as being the most recent item in the history.
  */
 void
-clipman_history_set_item_to_restore (ClipmanHistory *history,
-                                     const ClipmanHistoryItem *item)
+clipman_history_set_image_to_restore (ClipmanHistory *history,
+                                      const ClipmanHistoryItem *item)
 {
-  /* TODO Verify that the item exists in the history */
-  history->priv->item_to_restore = item;
+  history->priv->image_to_restore = item;
+
+  if (item != NULL && history->priv->reorder_items)
+    {
+      history->priv->items = g_slist_remove (history->priv->items, item);
+      history->priv->items = g_slist_prepend (history->priv->items, (gpointer) item);
+    }
 }
 
 /**
@@ -400,7 +405,7 @@ clipman_history_clear (ClipmanHistory *history)
 
   g_slist_free (history->priv->items);
   history->priv->items = NULL;
-  history->priv->item_to_restore = NULL;
+  history->priv->image_to_restore = NULL;
 
   g_signal_emit (history, signals[CLEAR], 0);
 }
@@ -508,7 +513,7 @@ static void
 clipman_history_init (ClipmanHistory *history)
 {
   history->priv = clipman_history_get_instance_private (history);
-  history->priv->item_to_restore = NULL;
+  history->priv->image_to_restore = NULL;
   history->priv->scale_factor = 1;
 }
 
@@ -539,6 +544,8 @@ clipman_history_set_property (GObject *object,
 
     case SAVE_ON_QUIT:
       priv->save_on_quit = g_value_get_boolean (value);
+      if (!priv->save_on_quit)
+        clipman_history_clear (CLIPMAN_HISTORY (object));
       break;
 
     case REORDER_ITEMS:
