@@ -31,7 +31,6 @@
 #include <libxfce4util/libxfce4util.h>
 #include <x11-clipboard-manager/daemon.h>
 
-#include <collector.h>
 #include <menu.h>
 #include <plugin.h>
 #include <history.h>
@@ -52,16 +51,6 @@ GtkWidget   *clipman_history_treeview_init             (MyPlugin  *plugin);
 static void  clipman_history_copy_or_paste_on_activate (MyPlugin  *plugin,
                                                         guint      paste_on_activate);
 
-typedef struct _ClipmanHistoryShutdownData ClipmanHistoryShutdownData;
-
-struct _ClipmanHistoryShutdownData
-{
-  gchar *text;
-  gboolean add_primary_clipboard;
-  gboolean do_copy;
-};
-
-ClipmanHistoryShutdownData shutdown_data = { NULL, FALSE, FALSE };
 
 
 static void
@@ -70,7 +59,6 @@ clipman_history_row_activated (GtkTreeView       *treeview,
                                GtkTreeViewColumn *column,
                                MyPlugin          *plugin)
 {
-  gboolean add_primary_clipboard;
   GtkWidget *window;
   GtkTreeSelection *selection;
   GtkTreeModel *model;
@@ -89,17 +77,9 @@ clipman_history_row_activated (GtkTreeView       *treeview,
   gtk_tree_model_get (model, &iter,
                       COLUMN_TEXT, &text,
                       -1);
-
-  /* Only update the primary clipboard if the setting "Sync mouse selections" is enabled */
-  g_object_get (G_OBJECT (plugin->collector), "add-primary-clipboard", &add_primary_clipboard, NULL);
-
-  shutdown_data.do_copy = TRUE;
-  shutdown_data.text = g_strdup (text);
-  shutdown_data.add_primary_clipboard = add_primary_clipboard;
+  g_object_set_data_full (G_OBJECT (plugin->app), "text", text, g_free);
 
   window = gtk_widget_get_toplevel (GTK_WIDGET (treeview));
-
-
   if (GTK_IS_WINDOW (window))
     gtk_dialog_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
 }
@@ -238,7 +218,6 @@ clipman_history_treeview_init (MyPlugin *plugin)
   GtkTreeIter iter;
   GtkTreePath *path;
   GtkWidget *entry, *scroll, *treeview, *box;
-  gboolean reverse_order = FALSE;
 
   box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_widget_set_margin_start (box, 6);
@@ -316,17 +295,11 @@ clipman_history_treeview_init (MyPlugin *plugin)
   gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
   /* Get the history and populate the treeview */
-  plugin->history = clipman_history_get ();
   list = clipman_history_get_list (plugin->history);
-
-  g_object_get (G_OBJECT (plugin->menu), "reverse-order", &reverse_order, NULL);
-  if (reverse_order)
-    list = g_slist_reverse (list);
-
   if (list == NULL)
     {
       gtk_list_store_insert_with_values (liststore, &iter, 0,
-                                         COLUMN_PREVIEW, _("Clipboard is empty"),
+                                         COLUMN_PREVIEW, _("History is empty"),
                                          COLUMN_TEXT, "",
                                          -1);
     }
@@ -363,22 +336,6 @@ clipman_history_treeview_init (MyPlugin *plugin)
   return box;
 }
 
-static void
-clipman_history_dialog_finalize (MyPlugin  *plugin,
-                                 GtkWidget *window)
-{
-
-  plugin_save (plugin);
-
-  gtk_widget_destroy (plugin->menu);
-  g_object_unref (plugin->channel);
-  g_object_unref (plugin->history);
-  gtk_widget_destroy (plugin->dialog);
-
-  g_slice_free (MyPlugin, plugin);
-  xfconf_shutdown ();
-}
-
 static void
 clipman_history_dialog_response (GtkWidget *dialog,
                                  gint       response_id,
@@ -391,7 +348,7 @@ clipman_history_dialog_response (GtkWidget *dialog,
   else if (response_id == GTK_RESPONSE_APPLY)
     clipman_history_search_entry_activate (GTK_ENTRY (plugin->entry), plugin);
   else
-    clipman_history_dialog_finalize (plugin, dialog);
+    gtk_widget_destroy (plugin->dialog);
 }
 
 static void
@@ -420,6 +377,13 @@ clipman_history_copy_or_paste_on_activate (MyPlugin *plugin,
   gtk_button_set_label (GTK_BUTTON (plugin->submit_button), button_text);
 }
 
+static void
+clipman_history_paste_on_activate_changed (MyPlugin *plugin)
+{
+  internal_paste_on_activate = xfconf_channel_get_uint (plugin->channel, "/tweaks/paste-on-activate", PASTE_INACTIVE);
+  clipman_history_copy_or_paste_on_activate (plugin, internal_paste_on_activate);
+}
+
 GtkWidget *
 clipman_history_dialog_init (MyPlugin *plugin)
 {
@@ -458,8 +422,7 @@ clipman_history_dialog_init (MyPlugin *plugin)
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_APPLY);
 #endif
   gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (plugin->submit_button)), "suggested-action");
-
-  clipman_history_copy_or_paste_on_activate (plugin, internal_paste_on_activate);
+  clipman_history_paste_on_activate_changed (plugin);
 
   g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (clipman_history_dialog_response), plugin);
 
@@ -467,14 +430,44 @@ clipman_history_dialog_init (MyPlugin *plugin)
 }
 
 static void
-clipman_history_activate (GtkApplication *app,
-                          gpointer        user_data)
+clipman_history_shutdown (GApplication *app,
+                          gpointer user_data)
 {
-  MyPlugin *plugin = g_slice_new0 (MyPlugin);
-  GtkWidget *box;
+  MyPlugin *plugin = user_data;
 
-  plugin->app = app;
-  xfconf_init (NULL);
+  plugin_save (plugin);
+
+  g_object_unref (plugin->channel);
+  g_object_unref (plugin->history);
+  g_slice_free (MyPlugin, plugin);
+
+  xfconf_shutdown ();
+}
+
+static gint
+clipman_history_command_line (GApplication *app,
+                              GApplicationCommandLine *command_line,
+                              gpointer user_data)
+{
+  MyPlugin *plugin;
+  GtkWidget *box;
+  GError *error = NULL;
+
+  if (g_application_command_line_get_is_remote (command_line))
+    {
+      g_application_activate (app);
+      return EXIT_SUCCESS;
+    }
+
+  if (!xfconf_init (&error))
+    {
+      g_critical ("Xfconf initialization failed: %s", error->message);
+      g_error_free (error);
+      return EXIT_FAILURE;
+    }
+
+  plugin = g_slice_new0 (MyPlugin);
+  plugin->app = GTK_APPLICATION (app);
 
   /* Bind all settings relevant for this application */
   plugin->channel = xfconf_channel_new_with_property_base ("xfce4-panel", "/plugins/clipman");
@@ -487,21 +480,16 @@ clipman_history_activate (GtkApplication *app,
                           G_TYPE_BOOLEAN, plugin->history, "save-on-quit");
   xfconf_g_property_bind (plugin->channel, "/tweaks/reorder-items",
                           G_TYPE_BOOLEAN, plugin->history, "reorder-items");
-
-  plugin->collector = clipman_collector_get ();
-  xfconf_g_property_bind (plugin->channel, "/settings/add-primary-clipboard",
-                          G_TYPE_BOOLEAN, plugin->collector, "add-primary-clipboard");
-
-  plugin->menu = clipman_menu_new ();
-  internal_paste_on_activate = xfconf_channel_get_uint (plugin->channel, "/tweaks/paste-on-activate", PASTE_INACTIVE);
-  xfconf_g_property_bind (plugin->channel, "/tweaks/reverse-menu-order",
-                          G_TYPE_BOOLEAN, plugin->menu, "reverse-order");
+  g_signal_connect_swapped (plugin->channel, "property-changed::/tweaks/paste-on-activate",
+                            G_CALLBACK (clipman_history_paste_on_activate_changed), plugin);
 
   /* Read the history from the cache file */
   plugin_load (plugin);
 
   /* Initialize dialog */
   plugin->dialog = clipman_history_dialog_init (plugin);
+  g_signal_connect_swapped (app, "activate", G_CALLBACK (gtk_window_present), plugin->dialog);
+  g_signal_connect (app, "shutdown", G_CALLBACK (clipman_history_shutdown), plugin);
 
   /* Initialize and pack history treeview */
   box = clipman_history_treeview_init (plugin);
@@ -509,25 +497,32 @@ clipman_history_activate (GtkApplication *app,
   gtk_widget_show_all (box);
 
   gtk_widget_show_all (plugin->dialog);
+
+  return EXIT_SUCCESS;
 }
 
-static gboolean
-clipman_history_clipman_daemon_running (void)
+static gint
+clipman_history_clipman_daemon_running (GApplication *app,
+                                        GVariantDict *options,
+                                        gpointer user_data)
 {
-  GtkApplication *app;
   GError *error = NULL;
 
-  app = gtk_application_new ("org.xfce.clipman", 0);
-
-  g_application_register (G_APPLICATION (app), NULL, &error);
-  if (error != NULL)
+  if (!g_application_register (app, NULL, &error))
     {
       g_warning ("Unable to register GApplication: %s", error->message);
       g_error_free (error);
-      error = NULL;
+      return EXIT_FAILURE;
     }
 
-  return g_application_get_is_remote (G_APPLICATION (app));
+  if (!g_application_get_is_remote (app))
+    {
+      g_warning ("Unable to find the primary instance org.xfce.clipman");
+      clipman_common_show_warning_dialog ();
+      return EXIT_FAILURE;
+    }
+
+  return EXIT_SUCCESS;
 }
 
 gint
@@ -536,69 +531,44 @@ main (gint argc, gchar *argv[])
   GtkApplication *app;
   int status;
   GtkClipboard *clipboard;
-  GError *error = NULL;
-
-  if (!clipman_history_clipman_daemon_running ())
-    {
-      g_warning ("The clipboard daemon is not running, exiting. You can launch it with 'xfce4-clipman'.");
-      clipman_common_show_warning_dialog ();
-      return FALSE;
-    }
+  const gchar *text;
 
   /* Setup translation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
-  app = gtk_application_new ("org.xfce.clipman.history", G_APPLICATION_FLAGS_NONE);
-
-  g_application_register (G_APPLICATION (app), NULL, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to register GApplication: %s", error->message);
-      g_error_free (error);
-      error = NULL;
-    }
-
-  if (g_application_get_is_remote (G_APPLICATION (app)))
-    {
-      g_warning ("%s already running", argv[0]);
-      return FALSE;
-    }
-
-  g_signal_connect (app, "activate", G_CALLBACK (clipman_history_activate), NULL);
+  /* check if clipman daemon is running */
+  app = gtk_application_new ("org.xfce.clipman", G_APPLICATION_FLAGS_NONE);
+  g_signal_connect (app, "handle-local-options", G_CALLBACK (clipman_history_clipman_daemon_running), NULL);
   status = g_application_run (G_APPLICATION (app), argc, argv);
+  g_object_unref (app);
+  if (status != EXIT_SUCCESS)
+    return EXIT_FAILURE;
+
+  /* run clipman-history app */
+  app = gtk_application_new ("org.xfce.clipman.history", G_APPLICATION_HANDLES_COMMAND_LINE);
+  g_signal_connect (app, "command-line", G_CALLBACK (clipman_history_command_line), NULL);
+  status = g_application_run (G_APPLICATION (app), argc, argv);
+  text = g_object_get_data (G_OBJECT (app), "text");
 
   /* doing copy and paste outside of g_main to assure window being already gone and
    * as in some occasions the implicit gtk_clipboard_store() lags for 10 seconds
    */
-  if (shutdown_data.do_copy)
+  if (text != NULL)
     {
       clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-      gtk_clipboard_set_text (clipboard, shutdown_data.text, -1);
+      gtk_clipboard_set_text (clipboard, text, -1);
 
-      if (shutdown_data.add_primary_clipboard)
-        {
-          clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-          gtk_clipboard_set_text (clipboard, shutdown_data.text, -1);
-        }
-      g_free (shutdown_data.text);
+      /* we are out of the g_main - so we have store() on our own */
+      gtk_clipboard_store (clipboard);
 
       /* Should be after gtk_widget_destroy() to assure focus being in the (previous) "target" window */
       if (internal_paste_on_activate != PASTE_INACTIVE)
         {
           cb_paste_on_activate (internal_paste_on_activate);
         }
-
-      clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-      /* we are out of the g_main - so we have store() on our own */
-      gtk_clipboard_store (clipboard);
-
-      if (shutdown_data.add_primary_clipboard)
-        {
-          clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-          /* we are out of the g_main - so we have store() on our own */
-          gtk_clipboard_store (clipboard);
-        }
     }
 
+  g_object_unref (app);
+
   return status;
 }
